


#include "Componenets/KOFCharacterMovementComponent.h"

#include "GameModes/KOFDefaultGameMode.h"

UKOFCharacterMovementComponent::UKOFCharacterMovementComponent()
{
	PrimaryComponentTick.bCanEverTick = true;
}

void UKOFCharacterMovementComponent::TickComponent(float DeltaTime, ELevelTick TickType,
	FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	AKOFBaseCharacter* Owner = Cast<AKOFBaseCharacter>(GetOwner());
	if(!Owner)
	{
		UE_LOG(LogTemp, Error, TEXT("AKOFCharacterMovementComponent::PostPhysicsTickComponent - could not get the owner"));
		return;
	}
	
	AKOFDefaultGameMode* GameMode = Cast<AKOFDefaultGameMode>(GetWorld()->GetAuthGameMode());
	if(!GameMode)
	{
		UE_LOG(LogTemp, Error, TEXT("AKOFCharacterMovementComponent::PostPhysicsTickComponent - could not get the game mode"));
		return;
	}
	
	AKOFBaseCharacter* Opponent = GameMode->GetCurrentOpponent(Owner->IsPossessedByPlayer1());
	if(Opponent)
	{
		float HalfYDistance = GameMode->GetMaxAllowedDistanceFromOpponent() / 2.0f;
		float YMidpoint = (Owner->GetActorLocation().Y + Opponent->GetActorLocation().Y) / 2.0f;

		float AdjustedY = FMath::Clamp(Owner->GetActorLocation().Y, YMidpoint - HalfYDistance, YMidpoint + HalfYDistance);
		
		FVector CurrentLoc = Owner->GetActorLocation();
		Owner->SetActorLocation(FVector(CurrentLoc.X, AdjustedY, CurrentLoc.Z));
	}

	// clamp to stage bounds
	FVector CurrentLoc = Owner->GetActorLocation();
	float HalfStageLength = GameMode->GetStageLength() / 2.0f;
	Owner->SetActorLocation(FVector(CurrentLoc.X,
									FMath::Clamp(CurrentLoc.Y, -HalfStageLength, HalfStageLength),
									CurrentLoc.Z));
}
